﻿using System;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Research.Kinect.Nui;
using System.Windows;

namespace KinectReception.Controls
{
    /// <summary>
    /// Interaction logic for VitruvianManControl.xaml
    /// </summary>
    public partial class VitruvianManControl : UserControl
    {
        private readonly SolidColorBrush _fill = new SolidColorBrush(Colors.Red);
        private readonly SolidColorBrush _defaultfill = new SolidColorBrush(Colors.Yellow);

        private readonly RepeatBehavior _r = new RepeatBehavior(int.MaxValue);
        private readonly Duration _duration = new Duration(new TimeSpan(0, 0, 0, 1));
        private DoubleAnimation _da;

        public VitruvianManControl()
        {
            this.InitializeComponent();
            _defaultfill.Opacity = 0.5;
            ClearnHighlight();
            
        }

        public void HighlightJoint(JointID sourceJoint)
        {
            ClearnHighlight();
            SetHighlight(sourceJoint);
        }

        public void HighlightJoint(JointID sourceJoint, JointID targetJoint)
        {
            ClearnHighlight();
            SetHighlight(sourceJoint);
            SetHighlight(targetJoint);
        }

        private void SetHighlight(JointID joint)
        {
            switch (joint)
            {
                case JointID.AnkleLeft: HighlightGrid(AnkleLeft); break;
                case JointID.AnkleRight: HighlightGrid(AnkleRight); break;
                case JointID.ElbowLeft:HighlightGrid(ElbowLeft); break;
                case JointID.ElbowRight:HighlightGrid(ElbowRight); break;
                case JointID.FootLeft:HighlightGrid(FootLeft); break;
                case JointID.FootRight:HighlightGrid(FootRight); break;
                case JointID.HandLeft:HighlightGrid(HandLeft); break;
                case JointID.HandRight: HighlightGrid(HandRight); break;
                case JointID.Head:HighlightGrid(Head); break;
                case JointID.HipCenter: HighlightGrid(HipCenter); break;
                case JointID.HipLeft: HighlightGrid(HipLeft); break;
                case JointID.HipRight: HighlightGrid(HipRight); break;
                case JointID.KneeLeft: HighlightGrid(KneeLeft); break;
                case JointID.KneeRight: HighlightGrid(KneeRight); break;
                case JointID.ShoulderCenter: HighlightGrid(ShoulderCenter); break;
                case JointID.ShoulderLeft: HighlightGrid(ShoulderLeft); break;
                case JointID.ShoulderRight: HighlightGrid(ShoulderRight); break;
                case JointID.Spine: HighlightGrid(Spine); break;
                case JointID.WristLeft: HighlightGrid(WristLeft); break;
                case JointID.WristRight: HighlightGrid(WristRight); break;
            }
        }

        void HighlightGrid(Grid grid)
        {
            var eli = grid.Children.OfType<Ellipse>().Single();
            eli.Fill = _fill;

            foreach (var rect in grid.Children.OfType<Rectangle>())
            {
                rect.Visibility = Visibility.Visible;
            }

            Blink(eli);
        }

        void Blink(UIElement ui)
        {
            _da = new DoubleAnimation(1, 0.5, _duration) { RepeatBehavior = _r };
            ui.BeginAnimation(UIElement.OpacityProperty, _da);
        }

        private void ClearnHighlight()
        {
            foreach (var grid in Joints.Children.OfType<Grid>())
            {
                var eli = grid.Children.OfType<Ellipse>().Single();
                eli.BeginAnimation(UIElement.OpacityProperty, null);
                eli.Fill = _defaultfill;

                foreach (var rect in grid.Children.OfType<Rectangle>())
                {
                    rect.Visibility = Visibility.Hidden;
                }
            }
        }
    }
}