﻿// Written by Shai Raiten: http://blogs.microsoft.co.il/blogs/shair/

using System;
using System.Collections;
using System.IO;
using KinectReception.Enums;

namespace KinectReception.Managers
{
    public static class VideoManager
    {
        /// <summary>
        /// Define the type of video you want to play
        /// If you ask for out video and there isn't one return null - Stop Video and
        ///         start showing Kinect Image
        /// If you ask for in video and there isn't one then return random video.
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
public static Uri GetVideo(VideoType type)
{
    if (string.IsNullOrEmpty(Properties.Settings.Default.VideosLibraryFolder) ||
        !Directory.Exists(Properties.Settings.Default.VideosLibraryFolder))
        return null;
    else
    {
        string value = null;
        switch (type)
        {
            case VideoType.In:
                value = Properties.Settings.Default.VideosLibraryInFile;
                return string.IsNullOrEmpty(value) || !File.Exists(value) ?
                    CollectRandomMovie() : new Uri(value);
            case VideoType.Out:
                value = Properties.Settings.Default.VideosLibraryOutFile;
                return string.IsNullOrEmpty(value) || !File.Exists(value) ?
                    null : new Uri(value);
            default:
                return CollectRandomMovie();
        }
    }
}

private static Uri CollectRandomMovie()
{
    var movies = new ArrayList();

    foreach (var filter in Properties.Settings.Default.VideoFilter)
        movies.AddRange(Directory.GetFiles(Properties.Settings.Default.VideosLibraryFolder,
            filter));

    if (movies.Count == 0) return null;

    var rnd = new Random();
    return new Uri(movies[rnd.Next(movies.Count)].ToString());
}
    }
}
